-- Class: Event System
-- SDK: Corona - 09/12/11
-- Author: Andrew Burch
-- Date: 16/04/2012
-- Site: http://www.newhighscore.net
-- Contact: andrew.burch@newhighscore.net
-- Notes: Local event listener system
--

local tablex = require('core.tablex')

-- Optimsation
local _tclear = tablex.clear
local _tinsert = table.insert
local _ipairs = ipairs


local EventSystem = {}
local EventSystem_mt = {__index = EventSystem}

function EventSystem.new()
	local eventSystem = {
		nextId = 1,
		eventList = {},
		eventListeners = {},
	}
	
	return setmetatable(eventSystem, EventSystem_mt)
end

function EventSystem:allocId()
	local id = self.nextId
	self.nextId = id + 1
	return id
end

function EventSystem:registerEventListener(obj, eventName, fn)
	local allEventListeners = self.eventListeners
	local specificEventListeners = allEventListeners[eventName]
	
	if not specificEventListeners then
		specificEventListeners = {}
		allEventListeners[eventName] = specificEventListeners
	end
	
	if not obj.eventListenerId then
		obj.eventListenerId = self:allocId()
	end
	
	local id = obj.eventListenerId
	specificEventListeners[id] = fn
end

function EventSystem:deregisterEventListeners(id)
	local eventListeners = self.eventListeners

	for _, listeners in _ipairs(eventListeners) do
		listeners[id] = nil
	end
end

function EventSystem:addEvent(event)
	_tinsert(self.eventList, event)
end

function EventSystem:processEvents(dt, time, params)
	local eventList = self.eventList
	local eventListeners = self.eventListeners
	
	for _, event in _ipairs(eventList) do
		local eventName = event.name
		local listeners = eventListeners[eventName]
		
		if listeners then
			for _, fn in _ipairs(listeners) do
				fn(event, dt, time, params)
			end
		end
	end
	
	self:clearEvents()
end

function EventSystem:clearEvents()
	_tclear(self.eventList)
end

function EventSystem:clearEventListeners()
	_tclear(self.eventListeners)
end


return EventSystem